#include <windows.h>
#include "webp/decode.h"
#include <cstdio>
/*
 * 000000は黒
 * FFFFFFは白
 * 左下から右上
 * B G R の順
 */

int getDIBFromWebP(uint8_t *webpdata, long filesize, BITMAPFILEHEADER *fileHeader, BITMAPINFOHEADER *infoHeader, uint8_t **data)
{
	int width, height;
	unsigned long bitLength, bitWidth;
	uint8_t *bitmapdata;
	*data = WebPDecodeBGR(webpdata, filesize, &width, &height);
	if ((width*3) % 4 == 0) {
		bitLength = width*3;
	} else {
		bitLength = width*3+(4-(width*3)%4);
	}
	bitWidth = width*3;
	bitmapdata = (uint8_t*)malloc(sizeof(uint8_t)*bitLength*height);
	memset(bitmapdata, '\0', bitLength*height);
	for (int i=0;i<height;i++) {
		memcpy(bitmapdata+i*bitLength, *data+(height-i-1)*bitWidth, bitWidth);
	}
	free(*data);
	*data = bitmapdata;
	memset(fileHeader, 0, sizeof(fileHeader));
	memset(infoHeader, 0, sizeof(infoHeader));

	fileHeader->bfType = 'M'*256+'B';
	fileHeader->bfSize = sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER)+sizeof(uint8_t)*bitLength*height;
	fileHeader->bfOffBits = sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER);
	fileHeader->bfReserved1 = 0;
	fileHeader->bfReserved2 = 0;

	infoHeader->biSize = 40;
	infoHeader->biWidth = width;
	infoHeader->biHeight = height;
	infoHeader->biPlanes = 1;
	infoHeader->biBitCount = 24;
	infoHeader->biCompression = 0;
	infoHeader->biSizeImage = fileHeader->bfSize;
	infoHeader->biXPelsPerMeter = infoHeader->biYPelsPerMeter = 0;
	infoHeader->biClrUsed = 0;
	infoHeader->biClrImportant = 0;
	return 0;
}

void saveDIB(char *outfile, BITMAPFILEHEADER *fileHeader, BITMAPINFOHEADER *infoHeader, uint8_t *data)
{
	FILE *fp;
	fp = fopen(outfile, "wb");
	if (fp == NULL) {
		return;
	}
	fwrite(fileHeader, sizeof(BITMAPFILEHEADER), 1, fp);
	fwrite(infoHeader, sizeof(BITMAPINFOHEADER), 1, fp);
	fwrite(data, sizeof(uint8_t), fileHeader->bfSize-fileHeader->bfOffBits, fp);
	fclose(fp);
}

BOOL IsSupportedEx(char *filename, char *data)
{
	char header[] = {'R', 'I', 'F', 'F', '8', 0x13, 0x00, 0x00, 'W', 'E', 'B', 'P', 'V', 'P', '8', ' '};
	for (int i=0;i<0x10;i++) {
		if (data[i] != header[i]) {
			return FALSE;
		}
	}
	return TRUE;
}

int main(int argc, char *argv[])
{
	char *webpfile, *outfile;
	BITMAPFILEHEADER fileHeader = {0};
	BITMAPINFOHEADER infoHeader = {0};
	FILE *fp;
	fpos_t filesize;
	uint8_t *data, *webpdata;
	if (argc > 2) {
		webpfile = argv[1];
		outfile = argv[2];
	} else {
		printf("Usage %s webpfile output\n", argv[0]);
		return 0;
	}
	if ((fp=fopen(webpfile, "rb"))==NULL) {
		printf("File open error:%s\n", webpfile);
		return 0;
	}
	fseek(fp, 0, SEEK_END);
	fgetpos(fp, &filesize);
	fseek(fp, 0, SEEK_SET);
	webpdata = (uint8_t*)malloc((sizeof uint8_t)*filesize);
	fread(webpdata, sizeof(uint8_t), filesize, fp);

	getDIBFromWebP(webpdata, filesize, &fileHeader, &infoHeader, &data);
	free(webpdata);

	saveDIB(outfile, &fileHeader, &infoHeader, data);
	free(data);
	return 0;
}
